//    Demo 6c: Name.java

//  A simple Name record type implementing the LinearlyOrdered interface.
//
//                by Charles W. Neville, 1998
//             Copyright Charles W. Neville, 1998
//
//  We have to redo the Name class from Demo 4, RecordAccess.java, so it is 
//  public and implements the LinearlyOrdered interface.
//
//  The LinearlyOrdered interface requires classes to implement methods gt, lt 
//  and eq.  The Name class from Demo 4 already implements those methods, so 
//  all we have to add is an announcement to the Java world that class Name 
//  implements interface LinearlyOrdered.

import java.io.*;
import LinearlyOrdered;  

// name record -- a person's name -- with bundled access methods.
public class Name implements LinearlyOrdered {

   String  first;
   char    middle;
   String  last;

   // read a Name record from a BufferedReader
   //
   // returns the record read -- returns null if there are no more records,
   // throws IOException
   //
   // input assumptions:
   //
   // (1) This method assumes each field of the record in the underlying file
   // is contained on a SEPARATE LINE, with NO LEADING OR TRAILING SPACES, as 
   // in the example below:
   //
   //  Joe
   //  M
   //  Bflspk
   //
   // (2) This method assumes the BufferedReader is positioned at the beginning
   // of the line containing the first name field of the current record, Joe in
   // the example above.
   //
   // unless these assumptions are adhered to rigorously, this method will not 
   // work correctly.  (It is highly sensitive to mistakes in the format of the 
   // input file.)
   public static Name readName(BufferedReader br) throws IOException {

      Name name = new Name();

      name.first = br.readLine();  // note that readLine() returns an 
                                   // entire string.
      if (name.first == null) return null;  // end of file -- return null

      name.middle = (char) br.read(); // note that read() returns a single 
                                      // char.
      br.readLine();                  // get to beginning of next line
      name.last = br.readLine();
 
      return name;

   } // readName

   // write a Name record to standard output
   public void writeName() {

        System.out.println(first);
        System.out.println(middle);
        System.out.println(last);
   } // writeName

   // is this Name record > Name record b?  Compare alphabetically, with last 
   // name first, then first name, then middle initial.  gt(b) returns true if 
   // this record  > record b, false otherwise.  Java silliness requires us to
   // parametrize gt as gt(LinearlyOrdered b) rather than gt(Name b).
   public boolean gt(LinearlyOrdered b) {

      Name c = (Name) b;  // more Java silliness, cast parameter b 
                          // to a Name object

      // because of return statements, can just use bunch of if's, don't need
      // if -- else if -- else.
      if (this.last.compareTo(c.last) > 0) return true;  // this says,
           // if( this.last > c.last) return true, but have to use compareTo
           // method in place of > operator.
      if (this.last.compareTo(c.last) < 0) return false;  // this says,
           // if( this.last < c.last) return false, but have to use compareTo
           // method in place of < operator.
      if (this.first.compareTo(c.first) > 0) return true;
      if (this.first.compareTo(c.first) < 0) return false;
      if (this.middle > c.middle) return true;
      if (this.middle < c.middle) return false;

      return false;  // two records are equal in this case.

   } // gt

   // is this Name record < Name record b?  Compare alphabetically, with last 
   // name first, then first name, then middle initial.  lt(b) returns true if 
   // this record  < record b, false otherwise.  Java silliness requires us to
   // parametrize lt as lt(LinearlyOrdered b) rather than lt(Name b).
   public boolean lt(LinearlyOrdered b) {

      Name c = (Name) b;  // more Java silliness, cast parameter b 
                          // to a Name object

      // because of return statements, can just use bunch of if's, don't need
      // if -- else if -- else.
      if (this.last.compareTo(c.last) < 0) return true;  // this says,
           // if( this.last < c.last) return true, but have to use compareTo
           // method in place of < operator.
      if (this.last.compareTo(c.last) > 0) return false;  // this says,
           // if( this.last > c.last) return false, but have to use compareTo
           // method in place of < operator.
      if (this.first.compareTo(c.first) < 0) return true;
      if (this.first.compareTo(c.first) > 0) return false;
      if (this.middle < c.middle) return true;
      if (this.middle > c.middle) return false;

      return false;  // two records are equal in this case.

   } // lt

   // is this Name record = Name record b?  eq(b) returns true if 
   // this record  = record b, false otherwise.
   public boolean eq(LinearlyOrdered b) {

      return !gt(b) && !lt(b);

   } // eq 


}  // class Name

